# frozen_string_literal: true

require 'aws-sdk-cloudfront'
require 'aws-sdk-ec2'
require 'aws-sdk-elasticloadbalancingv2'
require 'aws-sdk-elasticsearchservice'
require 'aws-sdk-rds'
require 'json'

# scan stuff
class Scanner
  # There is no logic in this scan that would pick up
  # services we are currently not using that may have
  # the potential to create new public endpoints.
  # for example, we do not use dynamodb but it could have public endpoints if
  # we did begin using it.
  #
  # services chosen to be scanned here are the intersection of what we use
  # and what I suspect has public ips.
  def truvantis_public_scan(aws_access_key_id: nil, aws_secret_access_key: nil)
    aws_access_key_id     = ENV['AWS_ACCESS_KEY_ID'] if aws_access_key_id.nil?
    aws_secret_access_key = ENV['AWS_SECRET_ACCESS_KEY'] if aws_secret_access_key.nil?

    ###########################################################################
    # cloudfront
    ###########################################################################

    report                              = {}
    report[:cloudfront]                 = {}
    report[:cloudfront][:distributions] = {}

    region = ENV['AWS_REGION'] ||
             ENV['AWS_DEFAULT_REGION'] ||
             'us-east-1'

    credentials = Aws::Credentials.new(aws_access_key_id, aws_secret_access_key)
    cf_client   = Aws::CloudFront::Client.new(region: region, credentials: credentials)
    cf_set      = cf_client.list_distributions.distribution_list.to_h

    # get the domain name which may be public
    # attach the arn for unique object identification as it may be useful to truvantis
    # attach aliases to potentially be scanned as well
    cf_set[:items].each do |item|
      report[:cloudfront][:distributions] = { (item[:id]).to_s => {
        domain: item[:domain_name],
        arn: item[:arn],
        aliases: item[:aliases]
      } }
    end

    # I purposefully did not include origin endpoints
    # as those will always be either an EIP, Instance, or LB
    # all of which are included in more downstream scans

    ###########################################################################
    # ec2
    ###########################################################################
    # any public instances and interfaces

    report[:ec2] = {}
    report[:ec2][:instances] = {}

    ec2_client = Aws::EC2::Client.new(region: region, credentials: credentials)

    inst_set = ec2_client.describe_instances.to_h[:reservations]
    inst_set.each do |reservation|
      reservation[:instances].each do |instance|
        # record instance public_ip
        report[:ec2][:instances][(instance[:instance_id]).to_s] = { public_ip: instance[:public_ip] }

        # record a few things about the instance that may be useful
        report[:ec2][:instances][(instance[:instance_id]).to_s] = \
          { public_dns_name: instance[:public_dns_name] }

        # instances might have network_interfaces attached
        next unless instance[:network_interfaces].any?

        # check for important things in each interface
        instance[:network_interfaces].each do |net_inter|
          next if net_inter[:association].nil?

          report[:ec2][:instances][(instance[:instance_id]).to_s] = \
            { network_interface: {
              carrier_ip: net_inter[:association][:carrier_ip],
              public_ip: net_inter[:association][:public_ip]
            } }
        end
      end
    end

    ###########################################################################
    # load balancers
    ###########################################################################
    # any public instances and interfaces

    report[:lb] = {}
    report[:lb][:load_balancers] = {}

    lb_client = Aws::ElasticLoadBalancingV2::Client.new(region: region, credentials: credentials)

    # I am including internal facing load balancers and
    # truvantis can filter out internals
    lb_set = lb_client.describe_load_balancers.to_h
    lb_set[:load_balancers].each do |lb|
      report[:lb][:load_balancers][(lb[:load_balancer_name]).to_s] = {
        dns_name: lb[:dns_name],
        scheme: lb[:scheme],
        type: lb[:type]
      }
      report[:lb][:load_balancers][(lb[:load_balancer_name]).to_s][:addresses] = []

      # gather actual ip addresses as well
      lb[:availability_zones].each do |zone|
        zone[:load_balancer_addresses].each do |addy|
          report[:lb][:load_balancers][(lb[:load_balancer_name]).to_s][:addresses] << addy[:ip_address]
        end
      end
    end

    ###########################################################################
    # elastic ips
    ###########################################################################
    # uses ec2 client and weird method describe_addresses
    report[:eips] = []

    # grab public ips for eips
    ec2_client.describe_addresses.to_h[:addresses].each do |addy|
      report[:eips] << addy[:public_ip] if addy[:public_ip]
    end

    ###########################################################################
    # network interfaces
    ###########################################################################
    # uses ec2 client and describe_network_interfaces

    report[:network_interfaces] = {}
    ec2_client.describe_network_interfaces.to_h[:network_interfaces].each do |net_inter|
      next if net_inter[:association].nil?

      report[:network_interfaces][(net_inter[:network_interface_id]).to_s] = {
        association: {
          public_ip: net_inter[:association][:public_ip],
          public_dns_name: net_inter[:association][:public_dns_name]
        }
      }

      # apparently private_ip_address associations can have public ips
      # grab any of those endpoints if any exist
      next unless net_inter[:private_ip_addresses].any?

      net_inter[:private_ip_addresses].each do |pia|
        next if report[:network_interfaces][(net_inter[:network_interface_id]).to_s][:private_ip_addresses].nil?
        next unless report[:network_interfaces][(net_inter[:network_interface_id]).to_s][:private_ip_addresses].any?

        report[:network_interfaces][(net_inter[:network_interface_id]).to_s]\
          [:private_ip_addresses] << {
            public_dns_name: pia[:association][:public_dns_name],
            public_ip: pia[:association][:public_ip]
          }
      end
    end

    ###########################################################################
    # elasticsearch as a service may have public endpoints
    ###########################################################################
    # Aws::ElasticsearchService and describe elasticsearch domains

    es_client = Aws::ElasticsearchService::Client.new(region: region, credentials: credentials)

    domain_names = []
    es_client.list_domain_names.to_h[:domain_names].each do |dn|
      domain_names << dn[:domain_name]
    end

    report[:es] = {}
    es_client.describe_elasticsearch_domains(domain_names: domain_names).to_h[:domain_status_list].each do |domain|
      report[:es][(domain[:domain_id]).to_s] = { endpoints: domain[:endpoints] }
    end

    ###########################################################################
    # rds
    ###########################################################################
    # databases may have public endpoints
    rds_client = Aws::RDS::Client.new(region: region, credentials: credentials)
    rds_set = rds_client.describe_db_instances.to_h
    unless rds_set[:db_instances].nil?
      if rds_set[:db_instances].any?
        report[:rds] = {}
        rds_set[:db_instances].each do |dbi|
          report[:rds][(dbi[:db_instance_arn]).to_s] = { endpoint: dbi[:endpoint] }
        end
      end
    end

    ###########################################################################
    # bonus: open security groups and their associated resources
    ###########################################################################

    ###########################################################################
    # bonus: open network acls
    ###########################################################################

    report
  end
end
